@@ -2,13 +2,11 @@ package ISG::ParseConfig;
 
 # TODO:
 # - _order for sections
-# - dynamically change grammar in _sub (_dyn ?)
-# - default values
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = 1.8;
+$VERSION = 1.9;
 
 sub new($$)
 {
@@ -66,6 +64,21 @@ sub _quotesplit($)
     return @items;
 }
 
+sub _deepcopy {
+        # this handles circular references on consecutive levels,
+        # but breaks if there are any levels in between
+        # the makepod() and maketmpl() methods have the same limitation
+        my $what = shift;
+        return $what unless ref $what;
+        for (ref $what) {
+                /^ARRAY$/ and return [ map { $_ eq $what ? $_ : _deepcopy($_) } @$what ];
+                /^HASH$/ and return { map { $_ => $what->{$_} eq $what ? 
+                                            $what->{$_} : _deepcopy($what->{$_}) } keys %$what };
+                /^CODE$/ and return $what; # we don't need to copy the subs
+        }
+        die "Cannot _deepcopy reference type @{[ref $what]}";
+}
+
 sub _check_mandatory($$$$)
 {
     my $self    = shift;
@@ -153,6 +166,23 @@ sub _search_section($$)
     return undef;
 }
 
+# fill in default values for this section
+sub _fill_defaults ($) {
+    my $self = shift;
+    my $g = $self->{grammar};
+    my $c = $self->{cfg};
+    if ($g->{_vars}) {
+        for my $var (@{$g->{_vars}}) {
+                next if exists $c->{$var};
+                my $value = $g->{$var}{_default}
+                  if exists $g->{$var}{_default};
+                next unless defined $value;
+                $c->{$var} = $value;
+        }
+    }
+
+}
+
 sub _next_level($$$)
 {
     my $self = shift;
@@ -174,7 +204,57 @@ sub _next_level($$$)
         return 0;
     }
     push @{$self->{grammar_stack}}, $self->{grammar};
-    $self->{grammar} = $self->{grammar}{$s};
+    if ($s =~ m|^/(.*)/$|) {
+        # for sections specified by a regexp, we create
+        # a new branch with a deep copy of the section 
+        # grammar so that any _dyn sub further below will edit
+        # just this branch
+
+        $self->{grammar}{$name} = _deepcopy($self->{grammar}{$s});
+
+        # put it at the head of the section list
+        $self->{grammar}{_sections} ||= [];
+        unshift @{$self->{grammar}{_sections}}, $name;
+    } 
+
+    # support for recursive sections
+    # copy the section syntax to the subsection
+
+    if ($self->{grammar}{_recursive} 
+        and grep { $_ eq $s } @{$self->{grammar}{_recursive}}) {
+        $self->{grammar}{$name}{_sections} ||= [];
+        $self->{grammar}{$name}{_recursive} ||= [];
+        push @{$self->{grammar}{$name}{_sections}}, $s;
+        push @{$self->{grammar}{$name}{_recursive}}, $s;
+        my $grammarcopy = _deepcopy($self->{grammar}{$name});
+        if (exists $self->{grammar}{$name}{$s}) {
+                # there's syntax for a variable by the same name too
+                # make sure we don't lose it
+                %{$self->{grammar}{$name}{$s}} = ( %$grammarcopy, %{$self->{grammar}{$name}{$s}} );
+        } else {
+                $self->{grammar}{$name}{$s} = $grammarcopy;
+        }
+    }
+
+    # this uses the copy created above for regexp sections 
+    # and the original for non-regexp sections (where $s == $name)
+    $self->{grammar} = $self->{grammar}{$name};
+
+    # support for inherited values
+    # note that we have to do this on the way down
+    # and keep track of which values were inherited
+    # so that we can propagate the values even further
+    # down if needed
+    my %inherited;
+    if ($self->{grammar}{_inherited}) {
+        for my $var (@{$self->{grammar}{_inherited}}) {
+                next unless exists $self->{cfg}{$var};
+                my $value = $self->{cfg}{$var};
+                next unless defined $value;
+                next if ref $value; # it's a section
+                $inherited{$var} = $value;
+        }
+    }
 
     # config context
     my $order;
@@ -191,19 +271,32 @@ sub _next_level($$$)
         $self->_make_error('section or variable already exists');
         return 0;
     }
-    $self->{cfg}{$name} = {};
+    $self->{cfg}{$name} = { %inherited }; # inherit the values
     push @{$self->{cfg_stack}}, $self->{cfg};
     $self->{cfg} = $self->{cfg}{$name};
 
+    # keep track of the inherited values here;
+    # we delete it on the way up in _prev_level()
+    $self->{cfg}{_inherited} = \%inherited; 
+
     # meta data for _mandatory test
     $self->{grammar}{_is_section} = 1;
     $self->{cfg}{_is_section}     = 1;
-    $self->{cfg}{_grammar}        = $s;
+
+    # this uses the copy created above for regexp sections 
+    # and the original for non-regexp sections (where $s == $name)
+    $self->{cfg}{_grammar}        = $name;
+
     $self->{cfg}{_order} = $order if defined $order;
 
     # increase level
     $self->{level}++;
 
+    # if there's a _dyn sub, apply it
+    if (defined $self->{grammar}{_dyn}) {
+        &{$self->{grammar}{_dyn}}($s, $name, $self->{grammar});
+    }
+
     return 1;
 }
 
@@ -211,6 +304,9 @@ sub _prev_level($)
 {
     my $self = shift;
 
+    # fill in the values from _default keywords when going up
+    $self->_fill_defaults;
+
     # section name
     if (defined $self->{section}) {
         if ($self->{section} =~ /\//) {
@@ -221,6 +317,9 @@ sub _prev_level($)
         }
     }
 
+    # clean up the _inherited hash, we won't need it anymore
+    delete $self->{cfg}{_inherited};
+
     # config context
     $self->{cfg} = pop @{$self->{cfg_stack}};
 
@@ -326,6 +425,10 @@ sub _set_variable($$$)
                         return 0;
                 }
         }
+        # if there's a _dyn sub, apply it
+        if (defined $g->{_dyn}) {
+                &{$g->{_dyn}}($key, $value, $self->{grammar});
+        }
     }
     $self->{cfg}{$key} = $value;
     return 1;
@@ -497,8 +600,14 @@ sub _parse_line($$$)
     }
     /^(\S+)\s*=\s*(.*)$/ and do {
         if (defined $self->{cfg}{$1}) {
-            $self->_make_error('variable already defined');
-            return 0;
+            if (exists $self->{cfg}{_inherited}{$1}) {
+                # it's OK to override any inherited values
+                delete $self->{cfg}{_inherited}{$1};
+                delete $self->{cfg}{$1};
+            } else {
+                $self->_make_error('variable already defined');
+                return 0;
+            }
         }
         $self->_set_variable($1, $2) or return 0;
         return 1;
@@ -722,9 +831,16 @@ sub parse($$)
     $self->{file_stack}    = [];
     $self->{line_stack}    = [];
 
+    # we work with a copy of the grammar so the _dyn subs may change it
+    local $self->{grammar} = _deepcopy($self->{grammar});
+
     $self->_parse_file($file) or return undef;
 
     $self->_goto_level(0, undef) or return undef;
+
+    # fill in the top level values from _default keywords
+    $self->_fill_defaults;
+
     $self->_check_mandatory($self->{grammar}, $self->{cfg}, undef)
       or return undef;
 
@@ -735,7 +851,6 @@ sub parse($$)
 1
 
 __END__
-
 =head1 NAME
 
 ISG::ParseConfig - Simple config parser
@@ -789,6 +904,15 @@ The sub-section can also be a regular expression denoted by the syntax '/re/',
 where re is the regular-expression. In case a regular expression is used, a
 sub-hash named with the same '/re/' must be included in this hash.
 
+=item _recursive
+
+Array containing the list of those sub-sections that are I<recursive>, ie.
+that can contain a new sub-section with the same syntax as themselves.
+
+The same effect can be accomplished with circular references in the
+grammar tree or a suitable B<_dyn> section subroutine (see below},
+so this facility is included just for convenience.
+
 =item _vars
 
 Array containing the list of variables (assignments) in this section.
@@ -798,6 +922,11 @@ Analogous to sections, regular expressions can be used.
 
 Array containing the list of mandatory sections and variables.
 
+=item _inherited
+
+Array containing the list of the variables that should be assigned the
+same value as in the parent section if nothing is specified here.
+
 =item _table
 
 Hash containing the table grammar (see Special Table Keys). If not specified,
@@ -824,6 +953,15 @@ defined.
 
 Describes what this section is about
 
+=item _dyn
+
+A subroutine reference (function pointer) that will be called when
+a new section of this syntax is encountered. The subroutine will get
+three arguments: the syntax of the section name (string or regexp), the
+actual name encountered (this will be the same as the first argument for
+non-regexp sections) and a reference to the grammar tree of the section.
+This subroutine can then modify the grammar tree dynamically.
+
 =back
 
 =head3 Special Variable Keys
@@ -846,6 +984,10 @@ first argument. If the function returns a defined value it is assumed that
 the test was not successful and an error is generated with the returned
 string as content.
 
+=item _default
+
+A default value that will be assigned to the variable if none is specified or inherited.
+
 =item _doc
 
 Describtion of the variable.
@@ -854,6 +996,17 @@ Describtion of the variable.
 
 A one line example for the content of this variable.
 
+=item _dyn
+
+A subroutine reference (function pointer) that will be called when the
+variable is assigned some value in the config file. The subroutine will
+get three arguments: the name of the variable, the value assigned and
+a reference to the grammar tree of this section.  This subroutine can
+then modify the grammar tree dynamically.
+
+Note that no _dyn() call is made for default and inherited values of
+the variable.
+
 =back
 
 =head3 Special Table Keys
@@ -1119,6 +1272,8 @@ S<Tobias Oetiker E<lt>oetiker@ee.ethz.chE<gt>>
  2002-10-10 ds 1.6 More verbatim _text sections
  2004-02-09 to 1.7 Added _example propperty for pod and template generation
  2004-08-17 to 1.8 Allow special input files like "program|"
+ 2005-01-10 ds 1.9 Implemented _dyn, _default, _recursive, and _inherited (Niko Tyni) 
+
 =cut
 
 # Emacs Configuration
