@@ -223,7 +223,7 @@ sub count_args {
 	my $self = shift;
 	my $count = shift;
 
-	$count = $self->{cfg}{Database}{pings} unless defined $count;
+	$count = $self->pings() unless defined $count;
 	return ("-n", $count);
 }
 
@@ -241,7 +241,7 @@ sub udp_arg {
 
 	my $udp = $target->{vars}{udp};
 	$udp = $self->{properties}{udp} unless defined $udp;
-	push @args, "-u" if (defined $udp and $udp ne "no" and $udp != 0);
+	push @args, "-u" if (defined $udp and $udp ne "no" and $udp ne "0");
 
 	return @args;
 }
@@ -264,6 +264,8 @@ sub make_commandline {
 	my $target = shift;
 	my $count = shift;
 
+	$count |= $self->pings($target);
+
 	my @args = $self->make_args($target);
 	my $host = $self->make_host($target);
 	push @args, $self->proto_args($target);
