@@ -18,7 +18,7 @@ point to your copy of the FPing program. If it is not installed on
 your system yet, you can get it from http://www.fping.com/.
 
 The (optional) packetsize option lets you configure the packetsize for the pings sent.
-The FPing manpage has the following to say on this toppic:
+The FPing manpage has the following to say on this topic:
 
 Number of bytes of ping data to send.  The minimum size (normally 12) allows
 room for the data that fping needs to do its work (sequence number,
@@ -64,10 +64,10 @@ sub new($$$)
 
         if ($return =~ m/bytes, ([0-9.]+)\sms\s+.*\n.*\n.*:\s+([0-9.]+)/ and $1 > 0){
             $self->{pingfactor} = 1000 * $2/$1;
-            print "### fping seems to report in ", $1/$2, " miliseconds\n";
+            print "### fping seems to report in ", $1/$2, " milliseconds\n";
         } else {
             $self->{pingfactor} = 1000; # Gives us a good-guess default
-            print "### assuming you are using an fping copy reporting in miliseconds\n";
+            print "### assuming you are using an fping copy reporting in milliseconds\n";
         }
     };
 
@@ -91,10 +91,12 @@ sub ping ($){
     return unless @{$self->addresses};
     my @bytes = () ;
     push @bytes, "-b$self->{properties}{packetsize}" if $self->{properties}{packetsize};
-    my $pid = open3($inh,$outh,$errh, 
+    my @cmd = (
                     $self->{properties}{binary}, @bytes,
-                    '-C', $self->{cfg}{Database}{pings}, '-q','-B1','-i10','-r1',
+                    '-C', $self->pings, '-q','-B1','-i10','-r1',
                     @{$self->addresses});
+    $self->do_debug("Executing @cmd");
+    my $pid = open3($inh,$outh,$errh, @cmd);
     $self->{rtts}={};
     while (<$errh>){
         chomp;
