@@ -9,7 +9,9 @@ probes::basefork - Yet Another Base Class for implementing SmokePing Probes
 =head1 OVERVIEW
 
 Like probes::basevars, but supports the probe-specific property `forks'
-to determine how many processes should be run concurrently.
+to determine how many processes should be run concurrently. The
+targets are pinged one at a time, and the number of pings sent can vary
+between targets.
 
 =head1 SYNOPSYS
 
@@ -20,11 +22,27 @@ to determine how many processes should be run concurrently.
  forks = 10 
  # how long does a single 'ping' take
  timeout = 10
+ # how many pings to send
+ pings = 10
 
  + MyOtherForkingProbe
  # we don't want any concurrent processes at all for some reason.
  forks = 1 
 
+ *** Targets ***
+
+ menu = First
+ title = First
+ host = firsthost
+ probe = MyForkingProbe
+
+ menu = Second
+ title = Second
+ host = secondhost
+ probe = MyForkingProbe
+ +PROBE_CONF
+ pings = 20
+
 =head1 DESCRIPTION
 
 Not all pinger programs support testing multiple hosts in a single go like
@@ -47,16 +65,20 @@ targets than this value, another round of forks is done after the first
 processes are finished. This continues until all the targets have been
 tested.
 
-The timeout in which each child has to finish is set to 5 seconds multiplied
-by the number oof 'pings'. You can set the base timeout differently if you
-want to, using the timeout property of the probe in the master config file
-(this again will be multiplied by the number of pings). The probe itself can
-also override the default by providing a TimeOut methode which returns an
-integer.
+The timeout in which each child has to finish is set to 5 seconds
+multiplied by the maximum number of 'pings' of the targets. You can set
+the base timeout differently if you want to, using the timeout property
+of the probe in the master config file (this again will be multiplied
+by the maximum number of pings). The probe itself can also override the
+default by providing a TimeOut method which returns an integer.
 
 If the child isn't finished when the timeout occurs, it 
 will be killed along with any processes it has started.
 
+The number of pings sent can be specified in the probe-specific variable
+'pings', and it can be overridden by each target in the 'PROBE_CONF'
+section.
+
 =head1 AUTHOR
 
 Niko Tyni E<lt>ntyni@iki.fiE<gt>
@@ -96,8 +118,8 @@ my @signame;
 die("Missing TERM signal?") unless exists $signo{TERM};
 die("Missing KILL signal?") unless exists $signo{KILL};
 
-sub ping_one {
-	croak "ping_one: this must be overridden by the subclass";
+sub pingone {
+	croak "pingone: this must be overridden by the subclass";
 }
 
 sub TimeOut {
@@ -113,10 +135,18 @@ sub ping {
 
 	my $forks = $self->{properties}{forks} || $DEFAULTFORKS;
 
-	my $timeout = ( $self->{properties}{timeout} || $self->TimeOut()) * $self->{cfg}{Database}{pings};
+	my $timeout = $self->{properties}{timeout};
+	unless (defined $timeout and $timeout > 0) {
+		my $maxpings = 0;
+		for (@targets) {
+			my $p = $self->pings($_);
+			$maxpings = $p if $p > $maxpings;
+		}
+		$timeout = $maxpings * $self->TimeOut();
+	}
 
         $self->{rtts}={};
-	$self->do_debug("forks $forks, timeout per ping $timeout, rounds $self->{cfg}{Database}{pings}");
+	$self->do_debug("forks $forks, timeout per target $timeout");
 
 	while (@targets) {
 		my %targetlookup;
@@ -200,6 +230,23 @@ sub ping {
 	}
 }
 
+# the "private" method that takes a "tree" argument is used by Smokeping.pm
+sub _pings {
+	my $self = shift;
+	my $tree = shift;
+	my $vars = $self->vars($tree);
+	return $vars->{pings} if defined $vars->{pings};
+	return $self->SUPER::pings();
+}
+
+# the "public" method that takes a "target" argument is used by the probes
+sub pings {
+	my $self = shift;
+	my $target = shift;
+	return $self->SUPER::pings() unless ref $target;
+	return $self->_pings($target->{tree});
+}
+
 sub ProbeDesc {
 	return "Probe that can fork and doesn't override the ProbeDesc method";
 }
